@@ -1,23 +1,21 @@
 '''
-Copyright (C) 2017-2021  Bryant Moscon - bmoscon@gmail.com
+Copyright (C) 2017-2022 Bryant Moscon - bmoscon@gmail.com
 
 Please see the LICENSE file for the terms and conditions
 associated with this software.
 '''
 from decimal import Decimal
 from collections import defaultdict
-from functools import partial
 import logging
-from typing import Callable, Dict, List, Tuple
+from typing import Dict, Tuple
 
 from yapic import json
 
-from cryptofeed.connection import AsyncConnection, WSAsyncConn
+from cryptofeed.connection import AsyncConnection, RestEndpoint, Routes, WebsocketEndpoint
 from cryptofeed.defines import BID, ASK, BUY, CANDLES, KRAKEN, L2_BOOK, SELL, TICKER, TRADES
 from cryptofeed.exceptions import BadChecksum
 from cryptofeed.feed import Feed
 from cryptofeed.symbols import Symbol
-from cryptofeed.util.split import list_by_max_items
 from cryptofeed.exchanges.mixins.kraken_rest import KrakenRestMixin
 from cryptofeed.types import OrderBook, Trade, Ticker, Candle
 
@@ -27,11 +25,12 @@ LOG = logging.getLogger('feedhandler')
 
 class Kraken(Feed, KrakenRestMixin):
     id = KRAKEN
+    websocket_endpoints = [WebsocketEndpoint('wss://ws.kraken.com', limit=20)]
+    rest_endpoints = [RestEndpoint('https://api.kraken.com', routes=Routes('/0/public/AssetPairs'))]
+
     valid_candle_intervals = {'1m', '5m', '15m', '30m', '1h', '4h', '1d', '1w', '15d'}
     candle_interval_map = {'1m': 1, '5m': 5, '15m': 15, '30m': 30, '1h': 60, '4h': 240, '1d': 1440, '1w': 10080, '15d': 21600}
     valid_depths = [10, 25, 100, 500, 1000]
-    symbol_endpoint = 'https://api.kraken.com/0/public/AssetPairs'
-    websocket_endpoint = 'wss://ws.kraken.com'
     websocket_channels = {
         L2_BOOK: 'book',
         TRADES: 'trade',
@@ -66,47 +65,27 @@ class Kraken(Feed, KrakenRestMixin):
     def __reset(self):
         self._l2_book = {}
 
-    def connect(self) -> List[Tuple[AsyncConnection, Callable[[None], None], Callable[[str, float], None]]]:
-        """
-        Per Kraken Tech Support, subscribing to more than 20 symbols in a single request can lead
-        to data loss. Furthermore, too many symbols on a single connection can cause data loss as well.
-        """
+    async def subscribe(self, conn: AsyncConnection):
         self.__reset()
-        ret = []
+        for chan, symbols in conn.subscription.items():
+            sub = {"name": chan}
+            if self.exchange_channel_to_std(chan) == L2_BOOK:
+                max_depth = self.max_depth if self.max_depth else 1000
+                if max_depth not in self.valid_depths:
+                    for d in self.valid_depths:
+                        if d > max_depth:
+                            max_depth = d
+                            break
 
-        def build(options: list):
-            subscribe = partial(self.subscribe, options=options)
-            conn = WSAsyncConn(self.address, self.id, **self.ws_defaults)
-            return conn, subscribe, self.message_handler, self.authenticate
+                sub['depth'] = max_depth
+            if self.exchange_channel_to_std(chan) == CANDLES:
+                sub['interval'] = self.candle_interval_map[self.candle_interval]
 
-        for chan in self.subscription:
-            symbols = list(self.subscription[chan])
-            for subset in list_by_max_items(symbols, 20):
-                ret.append(build((chan, subset)))
-
-        return ret
-
-    async def subscribe(self, conn: AsyncConnection, options: Tuple[str, List[str]] = None):
-        chan = options[0]
-        symbols = options[1]
-        sub = {"name": chan}
-        if self.exchange_channel_to_std(chan) == L2_BOOK:
-            max_depth = self.max_depth if self.max_depth else 1000
-            if max_depth not in self.valid_depths:
-                for d in self.valid_depths:
-                    if d > max_depth:
-                        max_depth = d
-                        break
-
-            sub['depth'] = max_depth
-        if self.exchange_channel_to_std(chan) == CANDLES:
-            sub['interval'] = self.candle_interval_map[self.candle_interval]
-
-        await conn.write(json.dumps({
-            "event": "subscribe",
-            "pair": symbols,
-            "subscription": sub
-        }))
+            await conn.write(json.dumps({
+                "event": "subscribe",
+                "pair": symbols,
+                "subscription": sub
+            }))
 
     async def _trade(self, msg: dict, pair: str, timestamp: float):
         """
